#include "StdAfx.h"
#include "WASRefreshTicket.h"
#include "ConvertUTF.h"
#include "MessengerActor.h"
#include "UtilFunction.h"
#include "GameTalkAPIGlobal.h"

CWASRefreshTicket::CWASRefreshTicket(CWASActor *pActor) : 
	CWASCommand(pActor), m_bReissueTicket(false)
{
}


CWASRefreshTicket::~CWASRefreshTicket(void)
{

}


void CWASRefreshTicket::ExecuteAsync()
{
	USES_UTF8_CONVERSION;

	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL)
	{
		return;
	}	

	m_bReissueTicket = false;

	WSTRING strURL = m_pActor->GetWASServerAddress();
	strURL += _T("/ne/RefreshAuthSession.php");

	m_httpFacade.session().SetURL(W2U8(strURL.c_str()),	pMessengerActor->GetMessengerTicket());
	m_httpFacade.session().SetMethod(HTTP_METHOD_POST);

	m_httpFacade.session().AddBodyVariable("userid", W2U8(pMessengerActor->GetUserID()));
	m_httpFacade.session().AddBodyVariable("ticket", pMessengerActor->GetMessengerTicket());
	m_httpFacade.session().AddBodyVariable("ostype", W2U8(_T("1")));

	INpMsgr *pNpMsgr = pMessengerActor->GetNpMsgr();
	char szVersion[1024] = {0,};
	pNpMsgr->GetAppandOSVersionString(szVersion, 1024);
	m_httpFacade.session().AddBodyVariable("version", szVersion);
	
	m_httpFacade.session().SetKeepRawHeader(TRUE);
	
	bool bSendResult = m_httpFacade.SendRequest(this);
	//NTALK_LOG->Debug(L"[LOGIN] WASREFRESHTICKET EXECUTED:[%d]", bSendResult);
}

void CWASRefreshTicket::ExecuteReIssueTicketAsync()
{
	USES_UTF8_CONVERSION;

	m_eError = wasError_None;

	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL)
	{
		return;
	}	

	m_bReissueTicket = true;

	WSTRING strURL = m_pActor->GetWASServerAddress();
	strURL += _T("/ne/user/ReissueTicket.php");

	m_httpFacade.session().SetURL(W2U8(strURL.c_str()));
	m_httpFacade.session().SetMethod(HTTP_METHOD_POST);

	m_httpFacade.session().AddBodyVariable("userid", W2U8(pMessengerActor->GetUserID()));

	// йȣ ȣȭ
	BYTE bytePwdCryptType = 0;
	BYTE biEncryptedData[1024] = {0,};
	USHORT usEncryptedPwdLength = sizeof(biEncryptedData);

	INpMsgr *pNpMsgr = pMessengerActor->GetNpMsgr();
	pNpMsgr->GetEncryptUserPassword(bytePwdCryptType, biEncryptedData, usEncryptedPwdLength);

	char szParam[1024] = {0,};
	memset(szParam, 0x00, sizeof(szParam));
	memcpy(szParam, biEncryptedData, usEncryptedPwdLength);

	m_httpFacade.session().AddBodyVariable("pw", szParam);
	m_httpFacade.session().AddBodyVariable("ostype", W2U8(_T("1")));

	char szVersion[1024] = {0,};
	pNpMsgr->GetAppandOSVersionString(szVersion, 1024);
	m_httpFacade.session().AddBodyVariable("version", szVersion);

	m_httpFacade.session().SetKeepRawHeader(TRUE);

	m_httpFacade.SendRequest(this);
}


bool CWASRefreshTicket::ParseAndEventFire(nfc::CIkXMLDom * pDom)
{
	if(pDom == NULL)
		return false;

	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL)
	{
		return false;
	}	

	return	ReadandAssignTicket(pDom);
}


// override in drived class
void CWASRefreshTicket::OnWasResponse(trid_t trid,
								  bool bSuccess,
								  int /*nWinINetErrorCode*/, 
								  int /*nHttpResponseCode*/, 
                                  LPCSTR /*szWasResCodeInHeader*/, 
                                  LPCSTR szWasResCodeInXml,
                                  nfc::CIkXMLDom * pDom)
{
	if (bSuccess)
	{
		if (ParseAndEventFire(pDom) == false)
		{
			m_eError = wasError_Protocol;
		}
		else
		{
			m_bReissueTicket = false;
		}
	}
	else
	{
		if (!m_bReissueTicket)
		{
			//Ƽ ſ   ű Ƽ ޱ޹ޱ⸦ õ.
			ExecuteReIssueTicketAsync();
			return;
		}
	}

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
		if((*it)->OnGTWASActor_ResponseRefreshTicket(m_pActor, GetID(), m_eError, m_strSession.c_str()) == false)
			break;

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}


void CWASRefreshTicket::OnRequestTimeout(CHttpFacade * /*pHttpF*/, DWORD /*dwSeconds*/)
{
	m_eError = wasError_Timeout;

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
		if((*it)->OnGTWASActor_ResponseRefreshTicket(m_pActor, GetID(), m_eError, m_strSession.c_str()) == false)
			break;

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}

bool CWASRefreshTicket::ReadandAssignTicket( nfc::CIkXMLDom * pDom )
{
	USES_UTF8_CONVERSION;

	bool result = false;
	nfc::IIkXMLAttribute * pAttr;
	nfc::IIkXMLNode * pNode;

	pNode = pDom->GetdocumentElement()->selectSingleNode("node");
	if(pNode == NULL)
		return result;

	pAttr = pNode->getAttribute("new_ticket");
	if(pAttr)
	{
		m_strSession  = U82W(pAttr->Gettext().c_str());
		result = true;
	}
	else
	{
		pAttr = pNode->getAttribute("reissued_ticket");

		if (pAttr)
		{
			m_strSession = U82W(pAttr->Gettext().c_str());
			result = true;
		}
	}

	return result;
}





CWASCheckClientNetwork::CWASCheckClientNetwork(CWASActor *pActor) : 
	CWASCommand(pActor),
	m_nCheckFailCnt(0)
{
}


CWASCheckClientNetwork::~CWASCheckClientNetwork(void)
{

}


void CWASCheckClientNetwork::ExecuteAsync()
{
	USES_UTF8_CONVERSION;

	m_eError = wasError_None;

	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL)
	{
		return;
	}	

	WSTRING strURL = m_pActor->GetWASServerAddress();
	strURL += _T("/NetCheck.php");

	m_httpFacade.session().SetURL(W2U8(strURL.c_str()),	pMessengerActor->GetMessengerTicket());
	m_httpFacade.session().SetMethod(HTTP_METHOD_GET);

	m_httpFacade.session().SetKeepRawHeader(TRUE);

	m_httpFacade.SendRequest(this);
}

bool CWASCheckClientNetwork::ParseAndEventFire(nfc::CIkXMLDom * pDom)
{
	if(pDom == NULL)
		return false;

	return true;
}


// override in drived class
void CWASCheckClientNetwork::OnWasResponse(trid_t trid,
									  bool bSuccess,
									  int /*nWinINetErrorCode*/, 
									  int /*nHttpResponseCode*/, 
									  LPCSTR /*szWasResCodeInHeader*/, 
									  LPCSTR szWasResCodeInXml,
									  nfc::CIkXMLDom * pDom)
{
	if (bSuccess)
	{
		if (ParseAndEventFire(pDom) == false)
			m_eError = wasError_Protocol;
	}

	if( m_eError == wasError_None )
		m_nCheckFailCnt = 0;
	else
		m_nCheckFailCnt++;

	GT_ASSERT(m_pActor);

	if( m_nCheckFailCnt >= 3 )
	{
		CGTObserved<IGTWASActorEventPrivate>::listobserver_t observers;
		m_pActor->GetObserversEx(&observers);
		CGTObserved<IGTWASActorEventPrivate>::listobserver_t::iterator it = observers.begin();

		for (; it != observers.end(); ++it)
		{
			IGTWASActorEventPrivate *pObserver = static_cast<IGTWASActorEventPrivate *>(*it);
			if(pObserver->OnGTWASActor_ResponseCheckClientNetwork(m_pActor, GetID(), m_eError) == false)
				break;
		}

		m_nCheckFailCnt = 0;
	}

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}


void CWASCheckClientNetwork::OnRequestTimeout(CHttpFacade * /*pHttpF*/, DWORD /*dwSeconds*/)
{
	m_eError = wasError_Timeout;

	m_nCheckFailCnt++;

	GT_ASSERT(m_pActor);

	if( m_nCheckFailCnt >= 3 )
	{
		CGTObserved<IGTWASActorEventPrivate>::listobserver_t observers;
		m_pActor->GetObserversEx(&observers);
		CGTObserved<IGTWASActorEventPrivate>::listobserver_t::iterator it = observers.begin();

		for (; it != observers.end(); ++it)
		{
			IGTWASActorEventPrivate *pObserver = static_cast<IGTWASActorEventPrivate *>(*it);
			if(pObserver->OnGTWASActor_ResponseCheckClientNetwork(m_pActor, GetID(), m_eError) == false)
				break;
		}

		m_nCheckFailCnt = 0;
	}

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}
